#!/usr/bin/env python
# -*- coding: utf-8 -*-
import unittest

from chibi_gob_mx import catalog
from chibi.atlas import Chibi_atlas


class Test_catalog(unittest.TestCase):
    def test_catalog_should_have_200( self ):
        response = catalog.get()
        self.assertEqual( response.status_code, 200  )
        self.assertIsInstance( response.native, list )

    def test_catalog_should_retrive_the_total_of_elements( self ):
        response = catalog.get()
        self.assertEqual( response.status_code, 200  )
        total_elements = list( response.native )
        self.assertEqual(
            len( total_elements ), response.pagination.total_elements )

    def test_should_have_the_expected_keys( self ):
        response = catalog.get()
        self.assertEqual( response.status_code, 200  )
        for n in response.native:
            self.assertIn( 'id', n )
            self.assertIn( 'count', n )
            self.assertIn( 'endpoint', n )
            self.assertIn( 'origin_url', n )
            self.assertIn( 'fields', n )


    def test_all_the_elements_should_be_dict( self ):
        response = catalog.get()
        self.assertEqual( response.status_code, 200  )
        keys = set()
        for n in response.native:
            self.assertIsInstance( n, Chibi_atlas )
            if 'variables' in n:
                v = set( n.variables )
                """
                for a in v:
                    if 'latitud' in a:
                        import pdb
                        pdb.set_trace()
                """
                keys |= v

        #for k in sorted( keys ):
            #print( k )



import copy

from chibi.atlas import Atlas
from chibi.chain import Chibi_chain
from chibi.metaphors import Book
from chibi.metaphors.book import End_book
from chibi_requests import Response

from .serializers import Catalog as Catalog_serializer


class Catalog( Response ):
    serializer = Catalog_serializer

    @property
    def native( self ):
        try:
            return self._native
        except AttributeError:
            from chibi_gob_mx.chibi_gob_mx import catalog
            native = super().native
            results = Atlas( native )
            page = copy.copy( self.pagination )
            try:
                page.next()
                catalog = catalog + page
                chain_params = dict(
                    next_obj=catalog,
                    retrieve_next=lambda x: catalog.get().native
                )
            except End_book:
                chain_params = dict( next_obj=None, retrieve_next=None )
            self._native = Chibi_chain( results, **chain_params )
            return self._native

    @property
    def pagination( self ):
        try:
            return self._pagination
        except AttributeError:
            native = self.parse_content_type()
            page = native.pagination
            page = Book(
                total_elements=page.total, page_size=page.pageSize,
                page=page.page, offset_dict={
                    'page': 'page', 'page_size': 'pageSize' } )

            self._pagination = page
            return self._pagination

    @property
    def native_is_many( self ):
        return True



from marshmallow import Schema, fields as f, pre_load, EXCLUDE


class Catalog( Schema ):
    id = f.String( data_key='_id', required=True )
    count = f.Integer( required=True )
    endpoint = f.String( required=True )
    origin_url = f.Url( data_key='url', required=True )
    fields = f.List(
        f.String, data_key='variables', required=False, missing=list )
    name = f.String( required=False )


    @pre_load( pass_many=True )
    def get_result( self, data, **kw ):
        self.many = True
        return data[ 'results' ]

    class Meta:
        unknown = EXCLUDE



from unittest import TestCase, skip
from chibi_gob_mx.profeco import precios


@skip( "no implementada" )
class Test_precios( TestCase ):
    def test_simple_call( self ):
        response = precios.get()
        self.assertEqual( response.status_code, 200  )
        self.assertFalse( True )
